import csv
import os
import re
import unittest

import hgvs.parser


def gcp_file_reader(fn):
    rdr = csv.DictReader(open(fn, 'r'), delimiter='\t')
    for rec in rdr:
        if rec['id'].startswith('#'):
            continue
        yield rec


class TestParseReal(unittest.TestCase):
    longMessage = True
    
    def setUp(self):
        self.parser = hgvs.parser.Parser()
        self._failed = []

        infilename = 'real_gcp.tsv'
        self._infile = os.path.join(os.path.dirname(__file__), 'data', infilename)

    def test_parse_hgvsg(self):
        key = 'HGVSg'
        omissions = [r'\(\d+\)$', r'copy\d+$', r'dup\d+$']
        self._parse_all(key, omissions)

    def test_parse_hgvsc(self):
        key = 'HGVSc'
        omissions = [r'copy\d+$', r'dup\d+$']
        self._parse_all(key, omissions)

    def test_parse_hgvsp(self):
        key = 'HGVSp'
        omissions = [r'dup', r'ext', r'Met1\?']
        self._parse_all(key, omissions)

    #
    # internal methods
    #

    def _parse_all(self, key, omissions = []):
        """Parse all tags of a given type

        :param key: record key corresponding to hgvs tag type (g, c ot p)
        :type key: str
        :param omissions: list of raw string literals corresponding to unsupported types
        :type omissions: list of raw string literals
        """

        # skip anything unsupported
        regexes = [re.compile(x) for x in omissions]

        outfilename = 'gcp_real_parsed_{}.out'.format(key)
        outfile = os.path.join(os.path.dirname(__file__), 'data', outfilename)
        with open(outfile, 'w') as out:
            out.write("id\t{}\n".format(key))
            for rec in gcp_file_reader(self._infile):
                if rec[key]:
                    hits = [True for x in regexes if len(x.findall(rec[key])) > 0]
                    if len(hits) == 0:
                        try:
                            var_x = self.parser.parse_hgvs_variant(rec[key])
                        except:
                            self._append_fail(out, rec['id'], rec[key])
            msg = "# failed: {}".format(len(self._failed))
            self.assertTrue(len(self._failed) == 0, msg)


    def _append_fail(self, out, row_id, hgvs_tag):
        self._failed.append((row_id, hgvs_tag))
        out.write("{}\t{}\n".format(row_id, hgvs_tag))

if __name__ == '__main__':
    unittest.main()

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



import recordtype

from hgvs.exceptions import HGVSError

class PosEdit( recordtype.recordtype( 'PosEdit', [('pos',None),('edit',None),('uncertain',False)] )):
    """
    represents a **simple** variant
    """
    
    def __str__(self):
        rv = str(self.edit) if self.pos is None else '{self.pos}{self.edit}'.format(self=self)
        if self.uncertain:
            if self.edit in ['0','']:
                rv = rv + '?'
            else:
                rv = '(' + rv + ')'
        return rv

    def set_uncertain(self):
        self.uncertain = True
        return self
    

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



import unittest

import hgvs.edit
from hgvs.exceptions import HGVSError

class Test_Edit(unittest.TestCase):
    def test_NARefAlt_exceptions(self):
        with self.assertRaises(HGVSError):
            edit = str(hgvs.edit.NARefAlt(None,None))

    def test_NARefAlt(self):
        self.assertEqual( str(hgvs.edit.NARefAlt('A','A'  ))  , '='             )
        self.assertEqual( str(hgvs.edit.NARefAlt('A','T'  ))  , 'A>T'           )
        self.assertEqual( str(hgvs.edit.NARefAlt('AA',None))  , 'delAA'         )
        self.assertEqual( str(hgvs.edit.NARefAlt(None,'TT'))  , 'insTT'         )
        self.assertEqual( str(hgvs.edit.NARefAlt('AA','T' ))  , 'delAAinsT'     )
        self.assertEqual( str(hgvs.edit.NARefAlt('A','TT' ))  , 'delAinsTT'     )


    def test_AARefAlt(self):
        self.assertEqual( str(hgvs.edit.AARefAlt('A','A'  ))  , '='             )
        self.assertEqual( str(hgvs.edit.AARefAlt('A','T'  ))  , 'Thr'           )
        self.assertEqual( str(hgvs.edit.AARefAlt('AA',None))  , 'del'           )
        self.assertEqual( str(hgvs.edit.AARefAlt(None,'TT'))  , 'insThrThr'     )
        self.assertEqual( str(hgvs.edit.AARefAlt('','T' ))  ,   'delinsThr'     )
        self.assertEqual( str(hgvs.edit.AARefAlt('AA','T' ))  , 'delinsThr'     )
        self.assertEqual( str(hgvs.edit.AARefAlt('A','TT' ))  , 'delinsThrThr'  )

        self.assertEqual( str(hgvs.edit.AARefAlt('A','T','fs*6'))  , 'Thrfs*6'  )

    def test_AASub(self):
        self.assertEqual( str(hgvs.edit.AASub('A','T'  ))  , 'Thr'           )
        self.assertEqual( str(hgvs.edit.AASub('A','T','fs*6'))  , 'Thrfs*6'  )


    def test_Dup(self):
        self.assertEqual( str(hgvs.edit.Dup())				, 'dup' 		)
        self.assertEqual( str(hgvs.edit.Dup('T'))			, 'dupT' 		)
        
    def test_Repeat(self):
        self.assertEqual( str(hgvs.edit.Repeat('CAG',12,34)), 'CAG(12_34)' 	)

    def test_Repeat_exceptions(self):
        with self.assertRaises(HGVSError):
            edit = str(hgvs.edit.Repeat('CAG',34,12))

if __name__ == '__main__':
    unittest.main()

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



#
# Mock test input source
#
from __future__ import with_statement
import csv

class MockInputSource():

    def __init__(self, in_file):
        self._mock_data = self._read_input(in_file)

    def fetch_gene_info(self, ac):
        pass

    def fetch_gene_transcripts(self, ac):
        pass

    def fetch_transcript_exons(self, ac):
        result = None
        data = self._mock_data.get(ac)
        if data:
            result = {'ord': 1,
                      't_start_i': 0,
                      't_end_i': data['cds_end_i'] - data['cds_start_i'],
                      't_seq_a': data['transcript_sequence']
            }

        return [result]

    def fetch_transcript_info(self,ac):
        result = None
        data = self._mock_data.get(ac)
        if data:     # interbase coordinates
            result = {'cds_start_i': data['cds_start_i'],
                      'cds_end_i': data['cds_end_i']}
        return result

    def get_tx_info(self, ac):
        return self.fetch_transcript_info(ac)

    def get_tx_exons(self, ac):
        return self.fetch_transcript_exons(ac)

    def get_tx_seq(self, ac):
        result = None
        data = self._mock_data.get(ac)
        if data:
            result = data['transcript_sequence']
        return result

    #
    # internal methods
    #

    def _read_input(self, in_file):
        """Dummy file of inputs

        :param in_file: path to input file of 2 cols (tab-delim); accession_number, sequence
        :type string
        :return dictionary of accession_number to sequence tags
        """
        result = {}
        with open(in_file, 'r') as f:
            reader = csv.DictReader(f, delimiter='\t')
            for row in reader:
                result[row['accession']] = {'transcript_sequence': row['transcript_sequence'],
                                                   'cds_start_i': int(row['cds_start_i']),
                                                   'cds_end_i': int(row['cds_end_i'])}

        return result


def main():
    pass


if __name__ == "__main__":
    main()

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



import unittest

import hgvs.hgvsposition

class Test_HGVSPosition(unittest.TestCase):
    def test_hgvsposition(self):
        var = hgvs.hgvsposition.HGVSPosition(
            seqref='NM_01234.5',
            type='c',
            pos=hgvs.location.Interval( hgvs.location.BaseOffsetPosition(base=12,offset=+34),
                                        hgvs.location.BaseOffsetPosition(base=56,offset=-78) ) )

        self.assertEqual( str(var) , 'NM_01234.5:c.12+34_56-78' )

if __name__ == '__main__':
    unittest.main()

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



class HGVSError(Exception):
    pass

class HGVSParseError(HGVSError):
    pass

class InvalidIntervalError(HGVSError):
    pass

###
#class UTAError(Exception):
#    pass
#
#class DatabaseError(UTAError):
#    pass
#
#class InvalidTranscriptError(UTAError):
#    pass
#
#
#
class InvalidHGVSVariantError(HGVSError):
    pass

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



import os
from pkg_resources import resource_filename

import parsley

import hgvs.edit
import hgvs.hgvsposition
import hgvs.location
import hgvs.posedit
import hgvs.variant
import hgvs.utils

from hgvs.exceptions import *

class Parser(object):
    __default_grammar_fn = resource_filename(__name__, 'data/hgvs.pymeta')

    def __init__(self,grammar_fn=__default_grammar_fn):
        self._grammar_fn = grammar_fn
        self._grammar = parsley.makeGrammar(open(grammar_fn,'r').read(),{'hgvs': hgvs})

        # define function attributes for each grammar rule, prefixed with 'parse_'
        # e.g., Parser.parse_c_interval('26+2_57-3') -> Interval(...)
        rules = [ m.replace('rule_','')
                  for m in dir(self._grammar._grammarClass)
                  if m.startswith('rule_') ]
        for rule_name in rules:
            att_name = 'parse_' + rule_name
            rule_fxn = self.__make_parse_rule_function(rule_name)
            self.__setattr__(att_name,rule_fxn)

    def __make_parse_rule_function(self,rule_name):
        # http://docs.python.org/2/reference/datamodel.html#object.__getattr__
        """
        This function returns a function that takes a string and returns the parsing result.
        """
        rule_fxn = lambda s: self._grammar(s).__getattr__(rule_name)()
        rule_fxn.func_doc = "parse string s using `%s' rule" % rule_name
        return rule_fxn

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



"""
hgvs.hgvsmapper
"""

import os,re,sys

from Bio.Seq import Seq
import recordtype

import hgvs.exceptions
import hgvs.variant
import hgvs.posedit
import hgvs.location
import hgvs.transcriptmapper
import hgvs.utils.altseq_to_hgvsp as altseq_to_hgvsp
import hgvs.utils.altseqbuilder as altseqbuilder
from hgvs.utils import reverse_complement
from hgvs.utils import chr_to_nc

class HGVSMapper(object):
    """
    Maps HGVS variants to and from g., r., c., and p. representations.
    All methods require and return objects of type hgvs.variant.Variant.
    """

    def __init__(self,bdi,cache_transcripts=False):
        self.bdi = bdi
        self.cache_transcripts = cache_transcripts
        self.__tm_cache = {}

    def hgvsg_to_hgvsc(self,var_g, ac, ref='GRCh37.p10'):
        """Given a genomic (g.) HGVS variant, return a transcript (c.) variant on the specified transcript.
        hgvs must be an HGVS-formatted variant or variant position.
        """

        if not (var_g.type == 'g'):
            raise hgvs.exceptions.InvalidHGVSVariantError('Expected a genomic (g.); got '+ str(var_g))

        tm = self._fetch_TranscriptMapper(ac=ac,ref=ref)
        
        pos_c = tm.hgvsg_to_hgvsc( var_g.posedit.pos )
        if isinstance(var_g.posedit.edit, hgvs.edit.NARefAlt):
            if tm.strand == 1:
                edit_c = var_g.posedit.edit
            else:
                edit_g = var_g.posedit.edit
                edit_c = hgvs.edit.NARefAlt(
                    ref = reverse_complement(edit_g.ref),
                    alt = reverse_complement(edit_g.alt),
                    )
        else:
            raise NotImplemented('Only NARefAlt types are currently implemented')

        var_c = hgvs.variant.SequenceVariant(ac=ac,
                                             type='c',
                                             posedit=hgvs.posedit.PosEdit( pos_c, edit_c ) )
        return var_c

    def hgvsg_to_hgvsr(self,var_g, ac, ref='GRCh37.p10'):
        """Given a genomic (g.) HGVS variant, return a transcript (r.) variant on the specified transcript.
        hgvs must be an HGVS-formatted variant or variant position.
        """

        if not (var_g.type == 'g'):
            raise hgvs.exceptions.InvalidHGVSVariantError('Expected a genomic (g.); got '+ str(var_g))

        tm = self._fetch_TranscriptMapper(ac=ac,ref=ref)

        pos_r = tm.hgvsg_to_hgvsr( var_g.posedit.pos )
        if isinstance(var_g.posedit.edit, hgvs.edit.NARefAlt):
            if tm.strand == 1:
                edit_r = var_g.posedit.edit
            else:
                edit_g = var_g.posedit.edit
                edit_r = hgvs.edit.NARefAlt(
                    ref = reverse_complement(edit_g.ref),
                    alt = reverse_complement(edit_g.alt),
                    )
        else:
            raise NotImplemented('Only NARefAlt types are currently implemented')

        var_r = hgvs.variant.SequenceVariant(ac=ac,
                                             type='r',
                                             posedit=hgvs.posedit.PosEdit( pos_r, edit_r ) )
        return var_r

    def hgvsr_to_hgvsg(self,var_r, ref='GRCh37.p10'):
        """Given an RNA (r.) HGVS variant, return a genomic (g.) variant from the inferred transcript.
        hgvs must be an HGVS-formatted variant or variant position.
        """

        if not (var_r.type == 'r'):
            raise hgvs.exceptions.InvalidHGVSVariantError('Expected a RNA (r.); got '+ str(var_r))

        tm = self._fetch_TranscriptMapper(ac=var_r.ac,ref=ref)

        pos_g = tm.hgvsr_to_hgvsg( var_r.posedit.pos )
        if isinstance(var_r.posedit.edit, hgvs.edit.NARefAlt):
            if tm.strand == 1:
                edit_r = var_r.posedit.edit
            else:
                edit_r = var_r.posedit.edit
                edit_g = hgvs.edit.NARefAlt(
                    ref = reverse_complement(edit_r.ref),
                    alt = reverse_complement(edit_r.alt),
                    )
        else:
            raise NotImplemented('Only NARefAlt types are currently implemented')

        # get NC accession for g.
        var_g = hgvs.variant.SequenceVariant(ac=chr_to_nc(tm.tx_info['chr']),
                                             type='g',
                                             posedit=hgvs.posedit.PosEdit( pos_g, edit_g ) )
        return var_g
    
    def hgvsc_to_hgvsg(self, var_c, ref='GRCh37.p10'):
        """Given a cDNA (c.) HGVS variant and an inferred transcript, return a genomic (g.) variant.
        hgvs must be an HGVS-formatted variant or variant position.
        """

        if not (var_c.type == 'c'):
            raise hgvs.exceptions.InvalidHGVSVariantError('Expected a cDNA (c.); got ' + str(var_c))

        tm = self._fetch_TranscriptMapper(ac=var_c.ac, ref=ref)

        pos_g = tm.hgvsc_to_hgvsg(var_c.posedit.pos)
        if isinstance(var_c.posedit.edit, hgvs.edit.NARefAlt):
            if tm.strand == 1:
                edit_g = var_c.posedit.edit
            else:
                edit_c = var_c.posedit.edit
                edit_g = hgvs.edit.NARefAlt(
                    ref = reverse_complement(edit_c.ref),
                    alt = reverse_complement(edit_c.alt),
                )
        else:
            raise NotImplemented('Only NARefAlt types are currently implemented')

        # get NC accession for g.
        var_g = hgvs.variant.SequenceVariant(ac=chr_to_nc(tm.tx_info['chr']),
                                             type='g',
                                             posedit=hgvs.posedit.PosEdit(pos_g, edit_g))
        return var_g

    def hgvsc_to_hgvsr(self, var_c, ref='GRCh37.p10'):
        """Given a cDNA (c.) HGVS variant and an inferred transcript, return an RNA (r.) variant.
        hgvs must be an HGVS-formatted variant or variant position
        """

        if not (var_c.type == 'c'):
            raise hgvs.exceptions.InvalidHGVSVariantError('Expected a cDNA (c.); got ' + str(var_c))

        tm = self._fetch_TranscriptMapper(ac=var_c.ac, ref=ref)
        pos_r = tm.hgvsc_to_hgvsr(var_c.posedit.pos)

        # not necessary to check strand
        if isinstance(var_c.posedit.edit, hgvs.edit.NARefAlt):
            edit_r = var_c.posedit.edit
        else:
            raise NotImplemented('Only NARefAlt types are currently implemented')

        var_r = hgvs.variant.SequenceVariant(ac=var_c.ac,
                                             type='r',
                                             posedit=hgvs.posedit.PosEdit( pos_r, edit_r ) )
        return var_r

    def hgvsr_to_hgvsc(self, var_r, ref='CRCh37.p10'):
        """Given a RNA (r.) HGVS variant and an inferred transcript, return an cDNA 
        hgvs must be an HGVS-formatted variant or variant position
        """

        if not (var_r.type == 'r'):
            raise hgvs.exceptions.InvalidHGVSVariantError('Expected a cDNA (c.); got ' + str(var_r))

        tm = self._fetch_TranscriptMapper(ac=var_r.ac, ref=ref)
        pos_c = tm.hgvsr_to_hgvsc(var_r.posedit.pos)

        # not necessary to check strand
        if isinstance(var_r.posedit.edit, hgvs.edit.NARefAlt):
            edit_c = var_r.posedit.edit
        else:
            raise NotImplemented('Only NARefAlt types are currently implemented')

        var_c = hgvs.variant.SequenceVariant(ac=var_r.ac,
                                             type='c',
                                             posedit=hgvs.posedit.PosEdit( pos_c, edit_c ) )
        return var_c

    def hgvsc_to_hgvsp(self, var_c, ac_p):
        """
        Converts a c. SequenceVariant to a p. SequenceVariant on the specified protein accession

        :param var_c: hgvsc tag
        :type var_c: SequenceVariant
        :param ac_p: protein accession
        :type ac_p: str
        :rtype: SequenceVariant

        """

        class RefTranscriptData(recordtype.recordtype('RefTranscriptData',
                                                      ['transcript_sequence', 'aa_sequence',
                                                       'cds_start', 'cds_stop', 'protein_accession'])):

            @classmethod
            def setup_transcript_data(cls, ac, ac_p, bdi, ref='GRCh37.p10'):
                """helper for generating RefTranscriptData from for hgvsc_to_hgvsp"""
                tx_info = bdi.get_tx_info(ac)
                tx_seq = bdi.get_tx_seq(ac)

                if tx_info is None or tx_seq is None:
                    raise hgvs.exceptions.HGVSError("Missing transcript data for accession: {}".format(ac))

                # use 1-based hgvs coords
                cds_start = tx_info['cds_start_i'] + 1
                cds_stop = tx_info['cds_end_i']

                # padding list so biopython won't complain during the conversion
                tx_seq_to_translate = tx_seq[cds_start - 1:cds_stop]
                if len(tx_seq_to_translate) % 3 != 0:
                    ''.join(list(tx_seq_to_translate).extend(['N']*((3-len(tx_seq_to_translate) % 3) % 3)))

                tx_seq_cds = Seq(tx_seq_to_translate)
                protein_seq = str(tx_seq_cds.translate())
                
                if ac_p is None:
                    # get_acs... will always return at least the MD5_ accession
                    ac_p = bdi.get_acs_for_protein_seq(protein_seq)[0]

                transcript_data = RefTranscriptData(tx_seq, protein_seq, cds_start,
                                                    cds_stop, ac_p)

                return transcript_data

        if not (var_c.type == 'c'):
            raise hgvs.exceptions.InvalidHGVSVariantError('Expected a cDNA (c.); got ' + str(var_c))

        reference_data = RefTranscriptData.setup_transcript_data(var_c.ac, ac_p, self.bdi)
        builder = altseqbuilder.AltSeqBuilder(var_c, reference_data)

        # TODO - handle case where you get 2+ alt sequences back; currently get list of 1 element
        # loop structure implemented to handle this, but doesn't really do anything currently.
        all_alt_data = builder.build_altseq()

        var_ps = []
        for alt_data in all_alt_data:
            builder = altseq_to_hgvsp.AltSeqToHgvsp(reference_data, alt_data)
            var_p = builder.build_hgvsp()
            var_ps.append(var_p)

        var_p = var_ps[0]

        return var_p

    ############################################################################
    ## Internal methods

    def _fetch_TranscriptMapper(self,ac,ref='GRCh37.p10'):
        """
        Get a new TranscriptMapper for the given transcript accession (ac),
        possibly caching the result.
        """
        try:
            tm = self.__tm_cache[ac]
        except KeyError:
            tm = hgvs.transcriptmapper.TranscriptMapper(self.bdi, ref = ref, ac = ac)
            if self.cache_transcripts:
                self.__tm_cache[ac] = tm
        return tm
 

if __name__ == '__main__':
    import hgvs.parser
    import hgvs.hgvsmapper
    from bdi.sources.uta0_sqlite import UTA0

    bdi = UTA0('/tmp/uta-0.0.4.db')
    hp = hgvs.parser.Parser()
    hm = hgvs.hgvsmapper.HGVSMapper(bdi, cache_transcripts=True)

    ref = 'GRCh37.p10'

    # From garcia.tsv:
    # AOAH    NM_001177507.1:c.1486G>A      
    hgvs_g = 'NC_000007.13:g.36561662C>T'
    hgvs_c = 'NM_001637.3:c.1582G>A'
    
    var_g = hp.parse_hgvs_variant(hgvs_g)
    var_c = hm.hgvsg_to_hgvsc( var_g, 'NM_001637.3' )

    print( str(var_g) + ' --> ' + str(var_c) )

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



import os
import pprint
import unittest

from parsley import ParseError

import hgvs.parser
from hgvs.exceptions import *


class Test_Position(unittest.TestCase):
    longMessage = True
    
    def setUp(self):
        self.parser = hgvs.parser.Parser()


    def test_parser_gauntlet(self):
        fn = os.path.join( os.path.dirname(__file__), 'data', 'gauntlet' )
        for var in open(fn,'r'):
            var = var.strip()
            if var.startswith('#') or var == '':
                continue
            v = self.parser._grammar(var).hgvs_variant()
            self.assertEqual( var, str(v), 'parse-format roundtrip failed:'+pprint.pformat(v.posedit) )


    def test_parser_reject(self):
        fn = os.path.join( os.path.dirname(__file__), 'data', 'reject' )
        for var in open(fn,'r'):
            var,msg = var.strip().split('\t')
            if var.startswith('#') or var == '':
                continue
            with self.assertRaises(ParseError):
                self.parser.parse_hgvs_variant(var)
                self.assertTrue(False, msg='expected HGVSParseError: %s (%s)' % (var,msg))


    def test_parser_posedit_special(self):
        # See note in grammar about parsing p.=, p.?, and p.0
        self.assertEqual( str(self.parser.parse_p_posedit('0')), '0' )
        self.assertEqual( str(self.parser.parse_p_posedit('0?')), '0?' )
        #self.assertEqual( str(self.parser.parse_p_posedit('(0)')), '0?' )

        self.assertEqual( str(self.parser.parse_p_posedit('?')), '?' )
        #self.assertEqual( str(self.parser.parse_p_posedit('??')), '?' )
        #self.assertEqual( str(self.parser.parse_p_posedit('(?)')), '?' )

        self.assertEqual( str(self.parser.parse_p_posedit('=')), '=' )
        #self.assertEqual( str(self.parser.parse_p_posedit('=?')), '(=)' )
        self.assertEqual( str(self.parser.parse_p_posedit('(=)')), '(=)' )

if __name__ == '__main__':
    unittest.main()

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



import unittest

import bdi.sources.uta0

from hgvs.exceptions import *
import hgvs.location
import hgvs.parser
import hgvs.projector

class TestHgvsProjector(unittest.TestCase):
    def setUp(self):
        self.bdi = bdi.sources.uta0.connect()
        self.ref = 'GRCh37.p10'
        self.hp = hgvs.parser.Parser()
        
    def tst_forward_and_backward(self,v1,v2):
        pj = hgvs.projector.Projector(self.bdi,self.ref,v1.ac,v2.ac)
        self.assertEqual( pj.project_variant_forward(v1), v2 )
        self.assertEqual( pj.project_variant_backward(v2), v1 )

    def test_rs201430561(self):
        # rs201430561 http://www.ncbi.nlm.nih.gov/projects/SNP/snp_ref.cgi?rs=201430561
        hgvs_c = ['NM_001197320.1:c.281C>T','NM_021960.4:c.740C>T','NM_182763.2:c.688+403C>T']
        var_c = [ self.hp.parse_hgvs_variant(h) for h in hgvs_c ]
        self.tst_forward_and_backward( var_c[0], var_c[1] )
        self.tst_forward_and_backward( var_c[0], var_c[2] )
        self.tst_forward_and_backward( var_c[1], var_c[2] )

    def test_bad_acs(self):
        hgvs_c = ['NM_001197320.1:c.281C>T','NM_021960.4:c.740C>T','NM_182763.2:c.688+403C>T']
        var_c = [ self.hp.parse_hgvs_variant(h) for h in hgvs_c ]
        pj = hgvs.projector.Projector(self.bdi,self.ref,var_c[0].ac,var_c[1].ac)
        # intentionally call p_v_f with variant on *destination* transcript, and vice versa
        self.assertRaises( RuntimeError, pj.project_variant_forward, var_c[1] )
        self.assertRaises( RuntimeError, pj.project_variant_backward, var_c[0] )


if __name__ == '__main__':
    unittest.main()



import os
import sys

from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages

# full path appears to be required for old (0.6.x?) versions of setuptools
root_dir = os.path.dirname(os.path.abspath(__file__))

with open(os.path.join(root_dir, 'doc/description.txt')) as f:
    long_description = f.read()

setup(
    license = "License :: OSI Approved :: Apache Software License",
    long_description = long_description,
    use_hg_version = True,
    zip_safe = False,

    author = 'HGVS Contributors',
    author_email = 'reece+hgvs@invitae.com',
    description = """HGVS Parser""",
    name = "hgvs",
    package_data = {'hgvs': ['data/*']},
    packages = find_packages(),
    url = 'https://bitbucket.org/invitae/hgvs',

    classifiers = [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Healthcare Industry",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python",
        "Topic :: Database :: Front-Ends",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
        ],
    keywords = [
        'bioinformatics',
        'computational biology',
        'genome variants',
        'genome variation',
        'genomic variants',
        'genomic variation',
        'genomics',
        'hgvs',
        ],

    install_requires = [
        'bdi',
        'biopython',
        'parsley',
        'recordtype',
        ],

    setup_requires = [
        'hgtools',
        'nose',
        ],

    tests_require = [
        'coverage',
        'nose',
        ],
    )

## <LICENSE>
## Copyright 2014 UTA Contributors (https://bitbucket.org/invitae/uta)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>



import warnings

import recordtype

class SequenceVariant( recordtype.recordtype('SequenceVariant', ['ac','type','posedit']) ):
    """
    represents a basic HGVS variant.  The only requirement is that each
    component can be stringified; for example, passing pos as either a string
    or an hgvs.location.CDSInterval (for example) are both intended uses
    """
    
    @property
    def seqref(self):
        warnings.warn('seqref is deprecated; use ac instead',DeprecationWarning,stacklevel=2)
        return self.ac

    def __str__(self):
        if self.ac is not None:
            return '{self.ac}:{self.type}.{self.posedit}'.format(self=self)
        else:
            return '{self.type}.{self.posedit}'.format(self=self)

## <LICENSE>
## Copyright 2014 HGVS Contributors (https://bitbucket.org/invitae/hgvs)
## 
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
## 
##     http://www.apache.org/licenses/LICENSE-2.0
## 
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
## </LICENSE>
